@@ -1,6 +1,9 @@
-#!/usr/bin/env python2
+#!/usr/bin/python3
 
-from SettingsWidgets import *
+import os
+
+from SettingsWidgets import SidePage
+from xapp.GSettingsWidgets import *
 from gi.repository import *
 
 PREF_MEDIA_AUTORUN_NEVER = "autorun-never"
@@ -14,6 +17,7 @@ CUSTOM_ITEM_OPEN_FOLDER = "cc-item-open-folder"
 
 MEDIA_HANDLING_SCHEMA = "org.cinnamon.desktop.media-handling"
 TERMINAL_SCHEMA = "org.cinnamon.desktop.default-applications.terminal"
+CALCULATOR_SCHEMA = "org.cinnamon.desktop.default-applications.calculator"
 
 PREF_CONTENT_TYPE = 0
 PREF_GEN_CONTENT_TYPE = 1
@@ -23,93 +27,115 @@ DEF_CONTENT_TYPE = 0
 DEF_LABEL = 1
 DEF_HEADING = 2
 
-preferred_app_defs = [
+preferred_app_defs = {}
+# Accessibility: Magnifier, Screen reader, Onscreen keyboard
+# Internet: Browser, Email Client, Instant messenger
+# Multimedia: Audio/Music player, Video player, Photos
+# Office: Word processor, Spreadsheet, Presentation, Document, Source code
+# System: file manager, Text editor, Terminal, Calculator
+
+preferred_app_defs[_("Accessibility")] = (
     # 1st mimetype is to let us find apps
     # 2nd mimetype is to set default handler for (so we handle all of that type, not just a specific format)
-    ( "x-scheme-handler/http",   "x-scheme-handler/http",      _("_Web") ),
-    ( "x-scheme-handler/mailto", "x-scheme-handler/mailto",    _("_Mail") ),
-    ( "application/msword",      "application/msword",       _("Documents") ),
+)
+
+preferred_app_defs[_("Internet")] = (
+    ( "x-scheme-handler/http",   "x-scheme-handler/http",    _("Web") ),
+    ( "x-scheme-handler/mailto", "x-scheme-handler/mailto",  _("Mail") ),
+)
+
+preferred_app_defs[_("Multimedia")] = (
+    ( "audio/x-vorbis+ogg",      "audio",                    _("Music") ),
+    ( "video/x-ogm+ogg",         "video",                    _("Video") ),
+    ( "image/jpeg",              "image",                    _("Photos") ),
+)
+
+preferred_app_defs[_("Office")] = (
+    ( "application/msword",      "application/msword",       _("Word") ),
+    ( "application/msexcel",     "application/msexcel",      _("Spreadsheet") ),
+    ( "application/pdf",         "application/pdf",          _("PDF") ),
+    ( "text/x-python",           "text/x-python",            _("Source Code") ),
+)
+
+preferred_app_defs[_("System")] = (
+    ( "inode/directory",         "inode/directory",          _("File Manager") ),
     ( "text/plain",              "text/plain",               _("Plain Text") ),
-    ( "audio/x-vorbis+ogg",      "audio",                    _("M_usic") ),
-    ( "video/x-ogm+ogg",         "video",                    _("_Video") ),
-    ( "image/jpeg",              "image",                    _("_Photos") ),
-    ( "text/x-python",           "text/x-python",          _("Source Code") ),
-]
+)
 
 mimetypes = {}
 mimetypes["audio"]=[
-  "audio/3gpp",
-  "audio/aac",
-  "audio/ac3",
-  "audio/flac",
-  "audio/m4a",
-  "audio/midi",
-  "audio/mp3",
-  "audio/mp4",
-  "audio/mp4a-latm",
-  "audio/mpeg",
-  "audio/mpeg3",
-  "audio/mpg",
-  "audio/ogg",
-  "audio/vorbis",
-  "audio/wav",
-  "audio/wave",
-  "audio/webm",
-  "audio/x-aac",
-  "audio/x-aiff",
-  "audio/x-flac",
-  "audio/x-mp3",
-  "audio/x-mpeg",
-  "audio/x-mpeg-3",
-  "audio/x-mpg",
-  "audio/x-ms-asf",
-  "audio/x-ms-wma",
-  "audio/x-ogg",
-  "audio/x-oggflac",
-  "audio/x-vorbis",
-  "audio/x-vorbis+ogg",
-  "audio/x-wav",
-  "audio/x-wavpack"
+    "audio/3gpp",
+    "audio/aac",
+    "audio/ac3",
+    "audio/flac",
+    "audio/m4a",
+    "audio/midi",
+    "audio/mp3",
+    "audio/mp4",
+    "audio/mp4a-latm",
+    "audio/mpeg",
+    "audio/mpeg3",
+    "audio/mpg",
+    "audio/ogg",
+    "audio/vorbis",
+    "audio/wav",
+    "audio/wave",
+    "audio/webm",
+    "audio/x-aac",
+    "audio/x-aiff",
+    "audio/x-flac",
+    "audio/x-mp3",
+    "audio/x-mpeg",
+    "audio/x-mpeg-3",
+    "audio/x-mpg",
+    "audio/x-ms-asf",
+    "audio/x-ms-wma",
+    "audio/x-ogg",
+    "audio/x-oggflac",
+    "audio/x-vorbis",
+    "audio/x-vorbis+ogg",
+    "audio/x-wav",
+    "audio/x-wavpack"
 ]
 
 mimetypes["video"]=[
-  "video/3gp",
-  "video/3gpp",
-  "video/divx",
-  "video/flv",
-  "video/mp4",
-  "video/mp4v-es",
-  "video/mpeg",
-  "video/msvideo",
-  "video/ogg",
-  "video/quicktime",
-  "video/vivo",
-  "video/vnd.divx",
-  "video/vnd.rn-realvideo",
-  "video/webm",
-  "video/x-anim",
-  "video/x-avi",
-  "video/x-flc",
-  "video/x-fli",
-  "video/x-flic",
-  "video/x-flv",
-  "video/x-m4v",
-  "video/x-matroska",
-  "video/x-mng",
-  "video/x-mpeg",
-  "video/x-mpeg2",
-  "video/x-ms-afs",
-  "video/x-ms-asf",
-  "video/x-ms-asx",
-  "video/x-ms-wm",
-  "video/x-ms-wmv",
-  "video/x-ms-wvx",
-  "video/x-ms-wvxvideo",
-  "video/x-msvideo",
-  "video/x-nsv",
-  "video/x-ogm+ogg",
-  "video/x-theora",
-  "video/x-theora+ogg"
+    "video/3gp",
+    "video/3gpp",
+    "video/divx",
+    "video/flv",
+    "video/mp4",
+    "video/mp4v-es",
+    "video/mpeg",
+    "video/msvideo",
+    "video/ogg",
+    "video/quicktime",
+    "video/vivo",
+    "video/vnd.divx",
+    "video/vnd.rn-realvideo",
+    "video/webm",
+    "video/x-anim",
+    "video/x-avi",
+    "video/x-flc",
+    "video/x-fli",
+    "video/x-flic",
+    "video/x-flv",
+    "video/x-m4v",
+    "video/x-matroska",
+    "video/x-mng",
+    "video/x-mpeg",
+    "video/x-mpeg2",
+    "video/x-ms-afs",
+    "video/x-ms-asf",
+    "video/x-ms-asx",
+    "video/x-ms-wm",
+    "video/x-ms-wmv",
+    "video/x-ms-wvx",
+    "video/x-ms-wvxvideo",
+    "video/x-msvideo",
+    "video/x-nsv",
+    "video/x-ogm+ogg",
+    "video/x-theora",
+    "video/x-theora+ogg"
 ]
 
 mimetypes["text/x-python"] = [
@@ -129,12 +155,20 @@ mimetypes["application/msword"] = [
     'application/vnd.ms-works', 'application/x-abiword',
 ]
 
+mimetypes["application/vnd.ms-excel"] = [
+    'application/vnd.ms-excel', 'application/vnd.oasis.opendocument.spreadsheet',
+    'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
+    'application/vnd.openxmlformats-officedocument.spreadsheetml.template',
+    'application/vnd.oasis.opendocument.spreadsheet-template',
+    'application/vnd.stardivision.calc',
+]
+
 removable_media_defs = [
-    ( "x-content/audio-cdda",       _("CD _audio") ,     _("Select an application for audio CDs")),
-    ( "x-content/video-dvd",        _("_DVD video"),     _("Select an application for video DVDs") ),
-    ( "x-content/audio-player",     _("_Music player"),  _("Select an application to run when a music player is connected") ),
-    ( "x-content/image-dcf",        _("_Photos"),        _("Select an application to run when a camera is connected") ),
-    ( "x-content/unix-software",    _("_Software"),      _("Select an application for software CDs") )
+    ( "x-content/audio-cdda",       _("CD audio") ,     _("Select an application for audio CDs")),
+    ( "x-content/video-dvd",        _("DVD video"),     _("Select an application for video DVDs") ),
+    ( "x-content/audio-player",     _("Music player"),  _("Select an application to run when a music player is connected") ),
+    ( "x-content/image-dcf",        _("Photos"),        _("Select an application to run when a camera is connected") ),
+    ( "x-content/unix-software",    _("Software"),      _("Select an application for software CDs") )
 ]
 
 other_defs = [
@@ -192,6 +226,9 @@ class MnemonicLabel(Gtk.Label):
         super(MnemonicLabel, self).__init__(label = "")
         self.set_text_with_mnemonic(text)
         self.set_mnemonic_widget(widget)
+        self.set_alignment(0.0, 0.5)
+        self.set_valign(Gtk.Align.START)
+        self.set_line_wrap(True)
 
 class DefaultAppChooserButton(Gtk.AppChooserButton):
     def __init__(self, content_type, gen_content_type):
@@ -204,11 +241,13 @@ class DefaultAppChooserButton(Gtk.AppChooserButton):
 
     def onChanged(self, button):
         info = button.get_app_info()
-        print "%s: " % info.get_name()
+
         if info:
+            print("%s: " % info.get_name())
+
             supported_mimetypes = info.get_supported_types()
             hardcoded_mimetypes = None
-            if self.generic_content_type in mimetypes.keys():
+            if self.generic_content_type in mimetypes:
                 hardcoded_mimetypes = mimetypes[self.generic_content_type]
 
             set_mimes = []
@@ -218,26 +257,27 @@ class DefaultAppChooserButton(Gtk.AppChooserButton):
                 for t in sorted(supported_mimetypes):
                     if t.startswith(self.generic_content_type):
                         if info.set_as_default_for_type (t):
-                            print "  Set as default for supported %s" % t
+                            print("  Set as default for supported %s" % t)
                             set_mimes.append(t)
                         else:
-                            print "  Failed to set as default application for '%s'" % t
+                            print("  Failed to set as default application for '%s'" % t)
 
             # Also assign mimes hardcoded in the mimetypes hashtable
             if hardcoded_mimetypes is not None:
                 for t in sorted(hardcoded_mimetypes):
                     if t not in set_mimes:
                         if info.set_as_default_for_type (t):
-                            print "  Set as default for hardcoded %s" % t
+                            print("  Set as default for hardcoded %s" % t)
                         else:
-                            print "  Failed to set as default application for '%s'" % t
+                            print("  Failed to set as default application for '%s'" % t)
 
             #Web
             if self.content_type == "x-scheme-handler/http":
-                if info.set_as_default_for_type ("x-scheme-handler/https") == False:
-                    print "  Failed to set '%s' as the default application for '%s'" % (info.get_name(), "x-scheme-handler/https")
+                if not info.set_as_default_for_type("x-scheme-handler/https"):
+                    print("  Failed to set '%s' as the default application for '%s'" % (info.get_name(), "x-scheme-handler/https"))
 
-class DefaultTerminalButton(Gtk.AppChooserButton): #TODO: See if we can get this to change the x-terminal-emulator default to allow it to be a more global change rather then just cinnamon/nemo
+class DefaultTerminalButton(Gtk.AppChooserButton):
+    #TODO: See if we can get this to change the x-terminal-emulator default to allow it to be a more global change rather then just cinnamon/nemo
     def __init__(self):
         super(DefaultTerminalButton, self).__init__()
         self.connect("changed", self.onChanged)
@@ -249,19 +289,76 @@ class DefaultTerminalButton(Gtk.AppChooserButton): #TODO: See if we can get this
         self.key_value = self.settings.get_string("exec")
         count_up = 0
 
-        while (self.this_item is not None and count_up < len(apps)):
+        while self.this_item is not None and count_up < len(apps):
             self.this_item = apps[count_up]
             cat_val = Gio.DesktopAppInfo.get_categories(self.this_item)
             exec_val = Gio.DesktopAppInfo.get_string(self.this_item, "Exec")
             name_val = Gio.DesktopAppInfo.get_string(self.this_item, "Name")
             icon_val = Gio.DesktopAppInfo.get_string(self.this_item, "Icon")
-            #terminals don't have mime types, so we check for "TerminalEmulator" under the "Category" key in desktop files
-            if (cat_val is not None and "TerminalEmulator" in cat_val):
-                #this crazy if statement makes sure remaining desktop file info is not empty, then prevents root terminals from showing, then prevents repeating terminals from trying to being added which leave a blank space and Gtk-WARNING's
-                if (exec_val is not None and name_val is not None and icon_val is not None and not "gksu" in exec_val and exec_val not in self.active_items):
+            # terminals don't have mime types, so we check for "TerminalEmulator" under the "Category" key in desktop files
+            if cat_val is not None and "TerminalEmulator" in cat_val:
+                # this crazy if statement makes sure remaining desktop file info is not empty, then prevents root terminals from showing, then prevents repeating terminals from trying to being added which leave a blank space and Gtk-WARNING's
+                if exec_val is not None and name_val is not None and icon_val is not None and not "gksu" in exec_val and exec_val not in self.active_items:
                     self.append_custom_item(exec_val, name_val, Gio.ThemedIcon.new(icon_val))
                     self.active_items.append(exec_val)
-                    if (self.key_value == exec_val):
+                    if self.key_value == exec_val:
+                        self.set_active_custom_item(self.key_value)
+            count_up += 1
+
+    def onChanged(self, button):
+        index_num = button.get_active()
+        command_key = self.active_items[index_num]
+        self.settings.set_string("exec", command_key)
+
+class TerminalExecArgEntry(Gtk.Entry):
+    def __init__(self):
+        super(TerminalExecArgEntry, self).__init__()
+
+        self.connect("changed", self.onChanged)
+
+        self.settings = Gio.Settings.new(TERMINAL_SCHEMA)
+        self.key_value = self.settings.get_string("exec-arg")
+
+        self.get_buffer().set_text(self.key_value, -1)
+
+        self.set_placeholder_text("exec-arg")
+        self.set_tooltip_text(_("Command-line option for your terminal to execute a passed-in command."))
+
+    def onChanged(self, entry):
+        self.settings.set_string("exec-arg", entry.get_buffer().get_text())
+
+class DefaultCalculatorButton(Gtk.AppChooserButton):
+    def __init__(self):
+        super(DefaultCalculatorButton, self).__init__()
+        apps = Gio.app_info_get_all()
+        self.this_item = []
+        self.active_items = []
+        self.settings = Gio.Settings.new(CALCULATOR_SCHEMA)
+        self.key_value = self.settings.get_string("exec")
+        self.connect("changed", self.onChanged)
+        count_up = 0
+
+        while self.this_item is not None and count_up < len(apps):
+            self.this_item = apps[count_up]
+            cat_val = Gio.DesktopAppInfo.get_categories(self.this_item)
+            exec_val = Gio.DesktopAppInfo.get_string(self.this_item, "Exec")
+            name_val = Gio.DesktopAppInfo.get_string(self.this_item, "Name")
+            icon_val = Gio.DesktopAppInfo.get_string(self.this_item, "Icon")
+            comment_val = Gio.DesktopAppInfo.get_string(self.this_item, "Comment")
+            #calculators don't have mime types, so we check for "Calculator" under the "Category" key in desktop files
+            if (cat_val is not None and "Calculator" in cat_val) or \
+               (exec_val is not None and "alculator" in exec_val.lower()) or \
+               (name_val is not None and "alculator" in name_val.lower()) or \
+               (comment_val is not None and "alculator" in comment_val.lower()):
+                #this if statement makes sure remaining desktop file info is not empty
+                if exec_val is not None and name_val is not None and icon_val is not None:
+                    if os.path.exists(icon_val):
+                        icon = Gio.FileIcon.new(Gio.File.new_for_path(icon_val))
+                    else:
+                        icon = Gio.ThemedIcon.new(icon_val)
+                    self.append_custom_item(exec_val, name_val, icon)
+                    self.active_items.append(exec_val)
+                    if self.key_value == exec_val:
                         self.set_active_custom_item(self.key_value)
             count_up += 1
 
@@ -276,6 +373,8 @@ class CustomAppChooserButton(Gtk.AppChooserButton):
         self.media_settings = media_settings
         content_type = self.get_content_type()
 
+        self.set_valign(Gtk.Align.CENTER)
+
         #fetch preferences for this content type
         (pref_start_app, pref_ignore, pref_open_folder) = self.getPreferences()
         pref_ask = not pref_start_app and not pref_ignore and not pref_open_folder
@@ -325,14 +424,14 @@ class CustomAppChooserButton(Gtk.AppChooserButton):
 
     def getPreference(self, settings_key):
         strv = self.media_settings.get_strv(settings_key)
-        return strv != None and self.get_content_type() in strv
+        return strv is not None and self.get_content_type() in strv
 
     def getPreferences(self):
         pref_start_app = self.getPreference( PREF_MEDIA_AUTORUN_X_CONTENT_START_APP)
         pref_ignore = self.getPreference(PREF_MEDIA_AUTORUN_X_CONTENT_IGNORE)
         pref_open_folder = self.getPreference(PREF_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER)
 
-        return (pref_start_app, pref_ignore, pref_open_folder)
+        return pref_start_app, pref_ignore, pref_open_folder
 
     def setPreference(self, pref_value, settings_key):
         array = self.media_settings.get_strv(settings_key)
@@ -348,9 +447,10 @@ class CustomAppChooserButton(Gtk.AppChooserButton):
         self.setPreference(pref_open_folder, PREF_MEDIA_AUTORUN_X_CONTENT_OPEN_FOLDER)
 
 class OtherTypeDialog(Gtk.Dialog):
-    def __init__(self, media_settings):
+    def __init__(self, media_settings, transient_parent):
         super(OtherTypeDialog, self).__init__(title = _("Other Media"),
-                                              transient_for = None,
+                                              transient_for = transient_parent,
+                                              border_width = 6,
                                               flags = 0)
         self.add_button(_("Close"), Gtk.ResponseType.OK)
 
@@ -375,7 +475,7 @@ class OtherTypeDialog(Gtk.Dialog):
         self.type_combo.set_active(False)
 
         table = ButtonTable(2)
-        table.addRow(_("_Type:"), self.type_combo)
+        table.addRow(_("Type:"), self.type_combo)
         self.table = table
 
         self.vbox.pack_start(ColumnBox(_("Select how other media should be handled"), table), True, True, 6)
@@ -393,6 +493,7 @@ class OtherTypeDialog(Gtk.Dialog):
         return True
 
     def getDescription(self, content_type):
+        description = None
         for d in other_defs:
             if content_type == d[DEF_CONTENT_TYPE]:
                 s = d[DEF_LABEL]
@@ -402,8 +503,8 @@ class OtherTypeDialog(Gtk.Dialog):
                     description = s
                 break
 
-        if description == None:
-            print "Content type '%s' is missing from the info panel" % content_type
+        if description is None:
+            print("Content type '%s' is missing from the info panel" % content_type)
             return Gio.content_type_get_description(content_type)
 
         return description
@@ -422,7 +523,7 @@ class OtherTypeDialog(Gtk.Dialog):
 
     def doHide(self):
         self.hide()
-        if self.application_combo != None:
+        if self.application_combo is not None:
             self.application_combo.destroy()
             self.application_combo = None
             self.table.forgetRow()
@@ -443,7 +544,7 @@ class OtherTypeDialog(Gtk.Dialog):
         heading = model.get_value(iter, 0)
 
         action_container = Gtk.HBox()
-        if self.application_combo != None:
+        if self.application_combo is not None:
             self.application_combo.destroy()
             self.table.forgetRow()
 
@@ -458,16 +559,16 @@ class Module:
     comment = _("Preferred Applications")
 
     def __init__(self, content_box):
-        keywords = _("media, defaults, applications, programs, removable, browser, email, calendar, music, videos, photos, images, cd, autostart, autoplay")
-        sidePage = SidePage(_("Preferred Applications"), "cs-default-applications", keywords, content_box, module=self)
+        keywords = _("media, defaults, applications, programs, removable, browser, email, calendar, music, videos, photos, images, cd, autoplay, favorite, apps")
+        sidePage = SidePage(_("Preferred Applications"), "cs-default-applications", keywords, content_box, 560, module=self)
         self.sidePage = sidePage
 
     def on_module_selected(self):
         if not self.loaded:
-            print "Loading Default module"
+            print("Loading Default module")
 
             self.media_settings = Gio.Settings.new(MEDIA_HANDLING_SCHEMA)
-            self.other_type_dialog = OtherTypeDialog(self.media_settings)
+            self.other_type_dialog = OtherTypeDialog(self.media_settings, self.sidePage.window)
 
             self.sidePage.stack = SettingsStack()
             self.sidePage.add_widget(self.sidePage.stack)
@@ -477,29 +578,56 @@ class Module:
             page = SettingsPage()
             self.sidePage.stack.add_titled(page, "preferred", _("Preferred applications"))
 
-
-            settings = page.add_section(_("Preferred applications"))
-
             size_group = Gtk.SizeGroup.new(Gtk.SizeGroupMode.HORIZONTAL)
 
-            for d in preferred_app_defs:
-                widget = SettingsWidget()
-                button = DefaultAppChooserButton(d[PREF_CONTENT_TYPE], d[PREF_GEN_CONTENT_TYPE])
-                label = MnemonicLabel(d[PREF_LABEL], button)
-                size_group.add_widget(button)
-                widget.pack_start(label, False, False, 0)
-                widget.pack_end(button, False, False, 0)
-                #Hide button if there are no apps
-                if not button.get_active():
-                    settings.add_row(widget)
+            for name in preferred_app_defs:
+                items = preferred_app_defs[name]
+                if len(items) > 0:
+                    settings = page.add_section(name)
+                    # size_group = Gtk.SizeGroup.new(Gtk.SizeGroupMode.HORIZONTAL)
+                    for item in items:
+                        widget = SettingsWidget()
+                        button = DefaultAppChooserButton(item[PREF_CONTENT_TYPE], item[PREF_GEN_CONTENT_TYPE])
+                        label = MnemonicLabel(item[PREF_LABEL], button)
+                        size_group.add_widget(button)
+                        widget.pack_start(label, False, False, 0)
+                        widget.pack_end(button, False, False, 0)
+                        #Hide button if there are no apps
+                        if not button.get_active():
+                            settings.add_row(widget)
+                    if name.lower() == "system":
+                        # Add Terminal and calculator to the "System" section
+
+                        # Calculator
+                        widget = SettingsWidget()
+                        button = DefaultCalculatorButton()
+                        label = MnemonicLabel(_("Calculator"), button)
+                        size_group.add_widget(button)
+                        widget.pack_start(label, False, False, 0)
+                        widget.pack_end(button, False, False, 0)
+                        settings.add_row(widget)
+
+                        # Terminal
+                        widget = SettingsWidget()
+                        button = DefaultTerminalButton()
+                        label = MnemonicLabel(_("Terminal"), button)
+                        entry_label = Gtk.Label(label="<i>%s</i>" % _("Arguments"), margin_end=4, use_markup=True)
+                        entry_label.get_style_context().add_class("dim-label")
+                        entry = TerminalExecArgEntry()
+
+                        entry_box = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, margin_top=6)
+                        entry_box.pack_start(entry_label, False, False, 0)
+                        entry_box.pack_start(entry, True, True, 0)
+
+                        box = Gtk.VBox()
+                        box.pack_start(button, False, False, 0)
+                        box.pack_start(entry_box, False, False, 0)
+                        size_group.add_widget(box)
+
+                        widget.pack_start(label, False, False, 0)
+                        widget.pack_end(box, False, False, 0)
+                        settings.add_row(widget)
 
-            widget = SettingsWidget()
-            button = DefaultTerminalButton()
-            label = MnemonicLabel(_("Te_rminal"), button)
-            size_group.add_widget(button)
-            widget.pack_start(label, False, False, 0)
-            widget.pack_end(button, False, False, 0)
-            settings.add_row(widget)
 
             # Removable media
 
@@ -511,7 +639,7 @@ class Module:
             switch.fill_row()
             page.add(switch)
 
-            settings = SettingsBox(_("Removable media"))
+            settings = SettingsSection(_("Removable media"))
             switch.revealer.add(settings)
             page.pack_start(switch.revealer, False, False, 0)
 
@@ -526,14 +654,15 @@ class Module:
                 widget.pack_end(button, False, False, 0)
                 settings.add_row(widget)
 
-            widget = SettingsWidget()
-            more = Gtk.Button.new_with_mnemonic(_("_Other Media..."))
-            more.connect("clicked", self.onMoreClicked)
-            widget.pack_start(more, True, True, 0)
-            settings.add_row(widget)
+            # FIXMEEEEEEEE??
+            button = Button(_("_Other Media...").strip("_"), self.onMoreClicked)
+            settings.add_row(button)
 
-    def onMoreClicked(self, button):
-        self.other_type_dialog.doShow(button.get_toplevel())
+    def onMoreClicked(self, widget):
+        self.other_type_dialog.doShow(widget.get_toplevel())
+
+    def _setParentRef(self, window):
+        self.sidePage.window = window
 
 class InvertedSwitch(SettingsWidget):
     def __init__(self, label, schema, key):
